"""Pluggable modules for Admin portal pages.

This module defines the following Class:

- `AdminPlugin`, the base object for plugin modules.  Not intended to be 
   directly instantiated, use the helper functions below.

Functions:

- `get_plugins`, get all plugins in a sorted list.
- `get_plugin`, get a plugin or create if not found.

How to use this module
======================

1. Add `admin_plugins` to the INSTALLED_APPS in settings.py

2. Import it: ``from admin_plugins import plugins``

3. Get a plugin module::
     plugin = plugins.get_plugin('wizards', _('Admin Wizards'))
     
4. Add links to the plugin module::
     plugin.add_link('caching_stats', _('View Cache'))

:Authors:
    - Bruce Kroeze <brucek@solidsitesolutions.com>
"""
"""
New BSD License
===============
Copyright (c) 2008, Bruce Kroeze http://gosatchmo.com

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of SolidSiteSolutions LLC, Zefamily LLC nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
__docformat__="restructuredtext"

from django.core import urlresolvers
import logging

log = logging.getLogger('admin_plugins')

_PLUGINS={}

class AdminPlugin(object):
    """A simple shell containing information about an admin sidebar module."""
    
    def __init__(self, slug, title, sort=0, template=None):
        self.title = title
        self.slug = slug
        self.template = template
        self._links = {}
        self.sort = sort
        
    def __cmp__(self, other):
        return cmp((self.sort, self.title), (other.sort, other.title))
        
    def __str__(self):
        return self.slug
        
    def add_link(self, key, title):
        self._links[key] = title
        
    @property
    def links(self):
        work = {}
        for key, title in self._links.items():
            url = urlresolvers.reverse(key)
            work[url] = title
        return work

def get_plugins():
    """Get all the `AdminPlugin` objects as a list"""
    plugins = _PLUGINS.values()
    plugins.sort()
    return plugins
    
def get_plugin(slug, title=None, sort=0, template=None):
    """Get or create an `AdminPlugin` object."""
    try:
        plugin = _PLUGINS[slug]
        plugin.sort = sort
        if title:
            plugin.title = title
        if template:
            plugin.template = template
    except KeyError:
        if not title:
            title = slug
        plugin = AdminPlugin(slug, title, sort, template)
        log.debug('Created AdminPlugin: %s', plugin)
        _PLUGINS[slug] = plugin
    
    return plugin

