"""Provides tags used to extend the django admin screens.

:Authors:
    - Bruce Kroeze
"""
"""
New BSD License
===============
Copyright (c) 2008, Bruce Kroeze http://solidsitesolutions.com

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of SolidSiteSolutions LLC, Zefamily LLC nor the names of its 
      contributors may be used to endorse or promote products derived from this 
      software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
__docformat__="restructuredtext"

from admin_plugins.plugins import get_plugins
from django import template
from django.conf import settings
from django.contrib.sites.models import Site
from django.template import loader
from django.utils.safestring import mark_safe

register = template.Library()

@register.inclusion_tag('admin/_admin_log.html', takes_context=False)
def admin_log(user):
    """Render the standard Django admin log"""
    return {
        'user' : user,
        }

@register.inclusion_tag('admin/_admin_plugin.html', takes_context=True)
def admin_plugin(context, adminplugin):
    """Render one plugin"""
    return {
        'context' : context,
        'plugin' : adminplugin
    }

@register.inclusion_tag('admin/_admin_plugin_list.html', takes_context=True)
def admin_plugin_list(context):
    """Render all the plugins."""
    return {
        'context' : context,
        'plugins' : get_plugins()
    }

@register.inclusion_tag('admin/_admin_plugin_links.html', takes_context=True)
def admin_plugin_links(context, adminplugin):
    """Render the links for a plugin."""
    return {
        'context' : context,
        'plugin' : adminplugin
    }
    
class AdminRenderNode(template.Node):
    def __init__(self, adminplugin):
        self.plugin = template.Variable(adminplugin)

    def render(self, context):
        plugin = self.plugin.resolve(context)
        if plugin.template:
            template = loader.get_template(plugin.template)
            return mark_safe(template.render(context))
        else:
            return ""

@register.tag
def admin_plugin_render_template(parser, token):
    """Render the template for a plugin, if any."""
    args = token.split_contents()
    if not len(args) == 2:
        raise template.TemplateSyntaxError("%r tag expected adminplugin, got: %s" % (args[0], args))

    tag, plugin = args
    return AdminRenderNode(plugin)
    
@register.simple_tag
def admin_sitename():
    site = None
    try:
        # if we have banjo running, then this can be multi-site aware.
        # and we want to return the current site's name
        from threaded_multihost import sites
        site = sites.site_by_host()
        
    except ImportError:
        pass

    if not site:
        site = Site.objects.get_current()
        
    if site:
        return site.name
        
    return settings.SITE_NAME

